#ifndef NULLMAILER__DEFINES__H__
#define NULLMAILER__DEFINES__H__

extern const char QUEUE_DIR[];
extern const char CONFIG_DIR[];
extern const char PROTOCOLS_DIR[];
extern const char BIN_DIR[];
extern const char SBIN_DIR[];

#define QUEUE_TEST_DIR "queue"
#define CONFIG_TEST_DIR "conf"
#define PROTOCOLS_TEST_DIR "protocols"
#define BIN_TEST_DIR "bin"
#define SBIN_TEST_DIR "sbin"

#endif /* NULLMAILER__DEFINES__H__ */
