// Copyright (C) 2018 Bruce Guenter <bruce@untroubled.org>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "fdbuf.h"
#include <errno.h>
#include <unistd.h>

bool fdobuf::seek(unsigned o)
{
  if(flags)
    return false;

  lock();
  unsigned buf_start = offset;
  unsigned buf_end = offset + buflength;
  if(o >= buf_start && o < buf_end) {
    bufpos = o - buf_start;
  }
  else {
    if(!nflush(false)) {
      unlock();
      return false;
    }
    if(lseek(fd, o, SEEK_SET) != (off_t)o) {
      errnum = errno;
      flags |= flag_error;
      unlock();
      return false;
    }
    offset = o;
  }
  count = 0;
  unlock();
  return true;
}
